from base_st_classes import *

class ReadSuite(NFSSuite):
    """Test operation 25: READ

##     FIXME: Adapt to protocol changes. 

##     FIXME: Add attribute directory and named attribute testing.
##     FIXME: Try reading a locked file. 

##     Equivalence partitioning:

##     Input Condition: current filehandle
##         Valid equivalence classes:
##             file(1)
##             named attribute(2)
##         Invalid equivalence classes:
##             dir(3)
##             special device files(4)
##             invalid filehandle(10)
##     Input Condition: stateid
##         Valid equivalence classes:
##             all bits zero(17)
##             all bits one(18)
##             valid stateid from open(19)
##         Invalid equivalence classes:
##             invalid stateid(20)
##     Input Condition: offset
##         Valid equivalence classes:2
##             zero(11)
##             less than file size(12)
##             greater than or equal to file size(13)
##         Invalid equivalence classes:
##             -
##     Input Condition: count
##         Valid equivalence classes:
##             zero(14)
##             one(15)
##             greater than one(16)
##         Invalid equivalence classes:
##             -
    """

    def setUp(self):
        NFSSuite.setUp(self)
        self.fh = None
        self.stateid = None
        # The contents of self.regfile
        self.data = """\
    Welcome to this NFS4 server.
    Enjoy.
    """

    def tearDown(self):
        self.ncl.close_file(self.fh, self.stateid, error=None)
        try: self.clean_dir(self.tmp_dir)
        except SkipException, e:
            pass

    #
    # Testcases covering valid equivalence classes.
    #
    def testSimpleRead(self):
        """READ from regular file with stateid=zeros

        Covered valid equivalence classes: 1, 12, 16, 17
        """
        eof, data = self.ncl.read_file(path=self.regfile)
        if eof != FALSE or data != self.data[:10]:
            self.fail("Data read incorrectly")

    def xtestReadAttr(self):
        """READ from named attribute

        Covered valid equivalence classes: 2, 12, 16, 18
        """
        # FIXME: Implement rest of testcase.
        self.skip("(TEST NOT IMPLEMENTED)")

    def testStateidOne(self):
        """READ with offset=2, count=1, stateid=ones

        Covered valid equivalence classes: 1, 12, 15, 18
        """
        stateid = stateid4(0xffffffffL,nfs4lib.long2opaque(0xffffffffffffffffffffffffL))
        eof, data = self.ncl.read_file(2, 1, stateid, path=self.regfile)
        if eof != FALSE or data != self.data[2:3]:
            self.fail("Data read incorrectly")

    def testWithOpen(self):
        """READ with offset=5, count=1000, stateid from OPEN

        Covered valid equivalence classes: 1, 12, 16, 19
        """

        self.init_connection()
        self.fh, self.stateid = self.ncl.open_confirm(self.regfile[:-1],
                                                      self.regfile[-1])
        putfhop = self.ncl.putfh_op(self.fh)
        readop = self.ncl.read(offset=5, count=1000, stateid=self.stateid)
        res = self.ncl.do_ops([putfhop, readop])
        self.assert_OK(res)
        obj = res.resarray[-1].arm.arm
        if obj.eof != TRUE or obj.data != self.data[5:]:
            self.fail("Data read incorrectly")

    def testLargeCount(self):
        """READ a large dataset"""
        # Create a large zeroed  file
        self.init_connection()
        attrs = nfs4lib.dict2fattr({FATTR4_SIZE: 10000000, FATTR4_MODE: 0644})
        self.fh, self.stateid = self.ncl.create_confirm(attrs)

        eof, data = self.ncl.read_file(0, 9000000)
        
    def testLargeOffset(self):
        """READ with offset outside file

        Covered valid equivalence classes: 1, 13, 16, 17
        """
        eof, data = self.ncl.read_file(1000, path=self.regfile)
        if eof != TRUE or data != "":
            self.fail("Data read incorrectly")

    def testZeroCount(self):
        """READ with count=0

        Covered valid equivalence classes: 1, 12, 14, 17
        """
        eof, data = self.ncl.read_file(5, 0, path=self.regfile)
        if eof != FALSE or data != "":
            self.fail("Data read incorrectly")
        
    #
    # Testcases covering invalid equivalence classes.
    #
    def testDirFh(self):
        """READ with (cfh)=directory should return NFS4ERR_ISDIR

        Covered invalid equivalence classes: 3
        """
        self.ncl.read_file(path=self.dirfile, error=[NFS4ERR_ISDIR])

    def testSpecials(self):
        """READ with (cfh)=device files should return NFS4ERR_INVAL

        Covered invalid equivalence classes: 4
        """
        for path in [self.blockfile,
                     self.charfile,
                     self.linkfile,
                     self.socketfile,
                     self.fifofile]:
            self.ncl.read_file(path=path, error=[NFS4ERR_INVAL])

    def testNoFh(self):
        """READ without (cfh) should return NFS4ERR_NOFILEHANDLE

        Covered invalid equivalence classes: 10
        """
        stateid = stateid4(0 , "")
        readop = self.ncl.read(stateid)
        res = self.ncl.do_ops([readop])
        self.assert_status(res, [NFS4ERR_NOFILEHANDLE])

    def testBadStateid(self):
        """READ with bad stateid should return NFS4ERR_BAD_STATEID

        Covered invalid equivalence classes: 20

        NOTE Mangling functions look into server opaque data, thus are
        specific to the CITI linux server
        """
        # Get a valid stateid
        self.init_connection()
        self.fh, self.stateid = self.ncl.open_confirm(self.regfile[:-1],
                                                      self.regfile[-1])
        # Mangle it
        badid = self.makeBadID(self.stateid)

        self.ncl.read_file(stateid=badid, path=self.regfile,
                           error=[NFS4ERR_BAD_STATEID])

    def testStaleStateid(self):
        """READ with stale stateid should return NFS4ERR_STALE_STATEID

        Covered invalid equivalence classes: 20

        NOTE Mangling functions look into server opaque data, thus are
        specific to the CITI linux server
        """
        # Get a valid stateid
        self.init_connection()
        self.fh, self.stateid = self.ncl.open_confirm(self.regfile[:-1],
                                                      self.regfile[-1])
        # Mangle it
        staleid = self.makeStaleID(self.stateid)

        self.ncl.read_file(stateid=staleid, path=self.regfile,
                           error=[NFS4ERR_STALE_STATEID])

    def testOpenMode(self):
        """READ with file opened in WRITE mode should return NFS4_OK or NFS4ERR_OPENMODE

        """

        self.init_connection()
        self.fh, self.stateid = self.ncl.open_confirm(self.regfile[:-1],
                                                      self.regfile[-1],
                                                      access=OPEN4_SHARE_ACCESS_WRITE)
        putfhop = self.ncl.putfh_op(self.fh)
        readop = self.ncl.read(offset=5, count=1000, stateid=self.stateid)
        res = self.ncl.do_ops([putfhop, readop])
        self.assert_status(res, [NFS4_OK, NFS4ERR_OPENMODE])
        if res.status==NFS4_OK:
            obj = res.resarray[-1].arm.arm
            if obj.eof != TRUE or obj.data != self.data[5:]:
                self.fail("Data read incorrectly")

